#include <avr/io.h>
#include <avr/interrupt.h>  /* for sei() */
#include <util/delay.h>     /* for _delay_ms() */
#include <avr/pgmspace.h>   /* required by usbdrv.h */
#include "usbdrv.h"

#define VENDOR_RQ_SET_LED    0
#define VENDOR_RQ_GET_SWITCH 1


/* ------------------------------------------------------------------------- */
/* ---------------------------- Utility Functions -------------------------- */
/* ------------------------------------------------------------------------- */

char in_c(char pin)
{
    // read logic level of port-C's pin
    DDRC &= ~(1<<pin);
    return ((PINC & (1<<pin))>>pin);    
} 
char in_b(char pin)
{
    // read logic level of port-B's pin
    DDRB &= ~(1<<pin);
    return ((PINB & (1<<pin))>>pin);    
} 
void out_c(char pin, char val)
{
    // write logic level to port-C's pin
    DDRC |= 1<<pin;
    if(val)
        PORTC |= 1<<pin;
    else
        PORTC &= ~(1<<pin);
}

/* ------------------------------------------------------------------------- */
/* ----------------------------- USB interface ----------------------------- */
/* ------------------------------------------------------------------------- */

uint16_t read_adc(uint8_t channel)
{
    // read analog value from specified channel of port C
    ADMUX = 0x40;
    ADMUX |= channel;
    ADCSRA = 0xC6;
    while ((ADCSRA & (1<<ADSC)))
        ;
    return ADCL + ADCH*256;
}

usbMsgLen_t usbFunctionSetup(uchar data[8])
{
    usbRequest_t *rq = (void *)data;
    static uchar dataBuffer[5];  /* must stay valid when usbFunctionSetup returns */

    if (rq->bRequest == VENDOR_RQ_SET_LED)
    {
        uint8_t led_no  = rq->wValue.bytes[0];
        uint8_t led_val = rq->wValue.bytes[1];
        out_c(led_no, led_val);
        return 0;
    }
    else if (rq->bRequest == VENDOR_RQ_GET_SWITCH)
    {
        dataBuffer[0] = !in_c(4);
        dataBuffer[1] = !in_b(0);
        dataBuffer[2] = !in_b(1);
        dataBuffer[3] = !in_b(4);
        dataBuffer[4] = !in_b(5);
        usbMsgPtr = dataBuffer;
        return 5;
    }
 
    return 0;   /* default for not implemented requests: return no data back to host */
}

/* ------------------------------------------------------------------------- */

int main(void)
{
    DDRC  = 0b00000111;  // PC0..PC2 = Output, PC3..PC5 = Input
    PORTC = 0b00000000;  // PC7..PC0 = 0
    PORTC |= (1 << 4);   // Enable pull-up at PC3
    DDRB  = 0b00000000;  // PC0..PC2 = Output, PC3..PC5 = Input
    PORTB = 0b00000000;  // PC7..PC0 = 0
    PORTB |= (1 << 0);   // Enable pull-up at PC3
    PORTB |= (1 << 1);
    PORTB |= (1 << 4);
    PORTB |= (1 << 5);
    DDRD  = 0b00001000;
    PORTD = 0b00000000;

    usbInit();
    usbDeviceDisconnect();  /* enforce re-enumeration, do this while interrupts are disabled! */
    _delay_ms(300);
    usbDeviceConnect();
    sei();
    for(;;){                /* main event loop */
        usbPoll();
    }
    return 0;
}

/* ------------------------------------------------------------------------- */
